@@ -3,6 +3,7 @@ import importlib
 import importlib.metadata
 import importlib.util
 import shutil
+import sys
 from contextlib import suppress
 from pathlib import Path
 
@@ -31,9 +32,16 @@ from sereto.keyring import get_password, set_password
 from sereto.models.settings import Settings
 from sereto.models.version import ProjectVersion
 from sereto.oxipng import run_oxipng
-from sereto.pdf import generate_pdf_finding_group, generate_pdf_report, generate_pdf_sow, generate_pdf_target
+from sereto.pdf import (
+    find_and_generate_pdf_finding_group,
+    generate_all_pdf_finding_groups,
+    generate_pdf_report,
+    generate_pdf_sow,
+    generate_pdf_target,
+)
 from sereto.project import Project, new_project
 from sereto.retest import add_retest
+from sereto.sereto_types import TypeProjectId
 from sereto.settings import load_settings_function
 from sereto.source_archive import (
     create_source_archive,
@@ -42,7 +50,6 @@ from sereto.source_archive import (
     retrieve_source_archive,
 )
 from sereto.tui.finding import launch_finding_tui
-from sereto.types import TypeProjectId
 from sereto.utils import copy_skel, replace_strings
 
 
@@ -587,6 +594,13 @@ def pdf() -> None:
 @click.option("-c", "--converter", help="Convert finding recipe")
 @click.option("-r", "--renderer", help="Build TeX finding recipe")
 @click.option("-v", "--version", help="Use config at specific version, e.g. 'v1.0'.")
+@click.option(
+    "-a",
+    "--all",
+    is_flag=True,
+    default=False,
+    help="Render all finding groups (mutually exclusive with --target-selector / --finding-group-selector).",
+)
 @click.pass_obj
 @validate_call
 def cli_pdf_finding_group(
@@ -596,6 +610,7 @@ def cli_pdf_finding_group(
     converter: str | None,
     renderer: str | None,
     version: ProjectVersion | None,
+    all: bool,
 ) -> None:
     """Generate a finding group PDF.\f
 
@@ -604,17 +619,25 @@ def cli_pdf_finding_group(
         target_selector: The target for which the finding group is being generated.
         finding_group_selector: The finding group to be generated.
         converter: The recipe for converting the findings.
-        renderer: The recipe for building TeX..
+        renderer: The recipe for building TeX.
         version: The version of the configuration to use. If None, the last version is used.
+        all: Flag to render all finding groups (exclusive with selectors).
     """
-    generate_pdf_finding_group(
-        project=ctx,
-        target_selector=target_selector,
-        finding_group_selector=finding_group_selector,
-        converter=converter,
-        renderer=renderer,
-        version=version,
-    )
+    # Exclusivity checks
+    if all and (target_selector is not None or finding_group_selector is not None):
+        raise SeretoValueError("--all cannot be used together with --target-selector or --finding-group-selector")
+
+    if all:
+        generate_all_pdf_finding_groups(project=ctx, converter=converter, renderer=renderer, version=version)
+    else:
+        find_and_generate_pdf_finding_group(
+            project=ctx,
+            target_selector=target_selector,
+            finding_group_selector=finding_group_selector,
+            converter=converter,
+            renderer=renderer,
+            version=version,
+        )
 
 
 @pdf.command(name="target")
@@ -872,7 +895,6 @@ def load_plugins() -> None:
     # Check if plugins are enabled
     if not settings.plugins.enabled:
         return
-
     # Get plugins directory
     plugins_dir = Path(
         replace_strings(text=settings.plugins.directory, replacements={"%TEMPLATES%": str(settings.templates_path)})
@@ -880,17 +902,36 @@ def load_plugins() -> None:
     if not plugins_dir.is_dir():
         raise SeretoPathError(f"Plugins directory not found: '{plugins_dir}'")
 
+    # Ensure plugins directory is in sys.path and plugins is a package
+    if not (plugins_dir / "__init__.py").exists():
+        raise SeretoPathError(f"Plugins directory '{plugins_dir}' is not a package (missing __init__.py)")
+    if str(plugins_dir) not in sys.path:
+        sys.path.insert(0, str(plugins_dir))
+
     # Load plugins from the directory
-    for file in plugins_dir.glob(pattern="*.py"):
+    for file in plugins_dir.iterdir():
         # Skip dunder files like __init__.py
         if file.name.startswith("__"):
             continue
 
+        if file.is_dir() and (file / "__init__.py").exists():
+            # It's a package
+            module_name = f"plugins.{file.name}"
+            module_path = file / "__init__.py"
+            submodule_search_locations = [str(file)]
+        elif file.suffix == ".py":
+            module_name = f"plugins.{file.name[:-3]}"
+            module_path = file
+            submodule_search_locations = None
+        else:
+            continue
+
         # Register commands
-        module_name = f"plugins.{file.name[:-3]}"
         try:
             # Create a module specification
-            spec = importlib.util.spec_from_file_location(module_name, file)
+            spec = importlib.util.spec_from_file_location(
+                module_name, module_path, submodule_search_locations=submodule_search_locations
+            )
             if spec is None or spec.loader is None:
                 Console().log(f"Failed to load plugin: {file.name}")
                 continue
