@@ -6,6 +6,7 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, Self
 
 import frontmatter  # type: ignore[import-untyped]
+import tomlkit
 from pydantic import DirectoryPath, FilePath, validate_call
 
 from sereto.convert import apply_convertor
@@ -18,7 +19,7 @@ from sereto.models.finding import (
     FindingTemplateFrontmatterModel,
     SubFindingFrontmatterModel,
 )
-from sereto.models.locator import LocatorModel
+from sereto.models.locator import LocatorModel, get_locator_types
 from sereto.models.settings import Render
 from sereto.models.version import ProjectVersion
 from sereto.risk import Risks
@@ -139,6 +140,7 @@ class FindingGroup:
         sub_findings: A list of sub-findings in the group.
         _target_locators: A list of locators used to find the target.
         _finding_group_locators: A list of locators defined on the finding group.
+        _show_locator_types: A list of locator types to return from the locators() property.
     """
 
     name: str
@@ -146,6 +148,7 @@ class FindingGroup:
     sub_findings: list[SubFinding]
     _target_locators: list[LocatorModel]
     _finding_group_locators: list[LocatorModel]
+    _show_locator_types: list[str]
 
     @classmethod
     @validate_call
@@ -180,21 +183,32 @@ class FindingGroup:
             sub_findings=sub_findings,
             _target_locators=target_locators,
             _finding_group_locators=group_desc.locators,
+            _show_locator_types=group_desc.show_locator_types,
         )
 
     def dumps_toml(self) -> str:
         """Dump the finding group to a TOML string."""
-        lines = [f'["{self.name}"]']
+        doc = tomlkit.document()
+        table = tomlkit.table()
+
         if self.explicit_risk is not None:
-            lines.append(f'risk = "{self.explicit_risk.value}"')
-        if len(self.sub_findings) == 1:
-            lines.append(f'findings = ["{self.sub_findings[0].uname}"]')
-        else:
-            lines.append("findings = [")
-            for sf in self.sub_findings:
-                lines.append(f'    "{sf.uname}",')
-            lines.append("]")
-        return "\n".join(lines)
+            table.add("risk", self.explicit_risk.value)
+
+        # show_locator_types (force inline)
+        slt_array = tomlkit.array()
+        slt_array.extend(self._show_locator_types)
+        table.add("show_locator_types", slt_array.multiline(False))
+
+        # findings (preserve order)
+        findings_array = tomlkit.array()
+        if len(self.sub_findings) > 1:
+            findings_array.multiline(True)
+        for sf in self.sub_findings:
+            findings_array.append(sf.uname)
+        table.add("findings", findings_array)
+
+        doc.add(self.name, table)
+        return tomlkit.dumps(doc).strip()
 
     @property
     def risk(self) -> Risk:
@@ -211,6 +225,8 @@ class FindingGroup:
     def locators(self) -> list[LocatorModel]:
         """Return a de-duplicated list of locators for the finding group.
 
+        Applies filtering from the `show_locator_types` attribute.
+
         Precedence (first non-empty wins):
         1. Explicit locators defined on the finding group
         2. All locators gathered from sub-findings
@@ -229,16 +245,19 @@ class FindingGroup:
             return result
 
         # 1. Explicit locators on the group
-        if self._finding_group_locators:
-            return _unique(self._finding_group_locators)
+        finding_group_locators = [loc for loc in self._finding_group_locators if loc.type in self._show_locator_types]
+        if len(finding_group_locators) > 0:
+            return _unique(finding_group_locators)
 
         # 2. Locators from sub-findings
-        sub_locators = _unique([loc for sf in self.sub_findings for loc in sf.locators])
-        if sub_locators:
-            return sub_locators
+        sub_finding_locators = _unique(
+            [loc for sf in self.sub_findings for loc in sf.locators if loc.type in self._show_locator_types]
+        )
+        if len(sub_finding_locators) > 0:
+            return sub_finding_locators
 
         # 3. Fallback to target locators
-        return _unique(self._target_locators)
+        return _unique([loc for loc in self._target_locators if loc.type in self._show_locator_types])
 
     @validate_call
     def filter_locators(self, type: str | Iterable[str]) -> list[LocatorModel]:
@@ -332,36 +351,33 @@ class Findings:
         variables: dict[str, Any] | None = None,
         overwrite: bool = False,
     ) -> None:
-        """Add a sub-finding from a template.
-
-        This will create a new finding group with a single sub-finding.
+        """Add a sub-finding from a template, creating a new finding group.
 
         Args:
-            templates: The path to the templates directory.
-            template_path: The path to the sub-finding template.
-            category: The category of the sub-finding.
-            name: The name of the sub-finding. If not provided, the name will use the default value from the template.
-            risk: The risk of the sub-finding. If not provided, the risk will use the default value from the template.
-            variables: A dictionary of variables to be used in the sub-finding template.
-            overwrite: If False, existing sub-finding will not be overwritten and new sub-finding with a random suffix
-                will be created. Otherwise, if True, existing sub-finding will be overwritten.
+            templates: Path to the templates directory.
+            template_path: Path to the sub-finding template.
+            category: Category of the sub-finding.
+            name: Name of the sub-finding group. Defaults to template name.
+            risk: Risk of the sub-finding. Defaults to template risk.
+            variables: Variables for the sub-finding template.
+            overwrite: If True, overwrite existing sub-finding; otherwise, create with random suffix.
         """
-        if variables is None:
-            variables = {}
-        overwriting = False
+        variables = variables or {}
 
-        # read template
+        # Load template metadata and content
         template_metadata = FindingTemplateFrontmatterModel.load_from(template_path)
         template_name = template_path.name.removesuffix(".md.j2")
-        _, content = frontmatter.parse(template_path.read_text(), encoding="utf-8")
+        _, content = frontmatter.parse(template_path.read_text(encoding="utf-8"))
 
-        # write sub-finding to findings directory
-        if (sub_finding_path := self.get_path(category=category, name=template_name)).is_file():
+        # Determine sub-finding path
+        sub_finding_path = self.get_path(category=category, name=template_name)
+        suffix = None
+
+        if sub_finding_path.is_file():
             if overwrite:
                 sub_finding_path.unlink()
-                overwriting = True
             else:
-                # Try up to 5 times to generate a unique random suffix
+                # Try to generate a unique filename with random suffix
                 for _ in range(5):
                     suffix = "".join(random.choices(string.ascii_lowercase + string.digits, k=5))
                     sub_finding_path = self.get_path(category=category, name=f"{template_name}_{suffix}")
@@ -369,10 +385,10 @@ class Findings:
                         break
                 else:
                     raise SeretoPathError(
-                        "sub-finding already exists and could not generate a unique filename after 5 attempts: "
-                        + str(sub_finding_path)
+                        f"sub-finding already exists and could not generate a unique filename: {sub_finding_path}"
                     )
 
+        # Prepare sub-finding frontmatter
         sub_finding_metadata = SubFindingFrontmatterModel(
             name=template_metadata.name,
             risk=template_metadata.risk,
@@ -380,28 +396,37 @@ class Findings:
             variables=variables,
             template_path=str(template_path.relative_to(templates)),
         )
+
+        # Write sub-finding file
         sub_finding_path.write_text(f"+++\n{sub_finding_metadata.dumps_toml()}+++\n\n{content}", encoding="utf-8")
 
-        if overwriting:
+        # If overwriting, nothing else to do
+        if overwrite:
             return
 
-        # load the created sub-finding
+        # Load the created sub-finding
         sub_finding = SubFinding.load_from(path=sub_finding_path, templates=templates)
 
-        # prepare finding group
+        # Determine group name
+        group_name = name or sub_finding.name
+        if suffix:
+            group_name = f"{group_name} {suffix}"
+
+        # Create finding group
         group = FindingGroup(
-            name=name or sub_finding_metadata.name,
+            name=group_name,
             explicit_risk=risk,
             sub_findings=[sub_finding],
             _target_locators=self.target_locators,
             _finding_group_locators=[],
+            _show_locator_types=get_locator_types(),
         )
 
-        # write the finding group to findings.toml
+        # Append to findings.toml
         with self.config_file.open("a", encoding="utf-8") as f:
             f.write(f"\n{group.dumps_toml()}\n")
 
-        # add to loaded finding groups
+        # Add to loaded groups
         self.groups.append(group)
 
     @validate_call
