@@ -4,14 +4,14 @@ from pathlib import Path
 from typing import Any, Literal, Self
 
 import frontmatter  # type: ignore[import-untyped]
-import tomli_w
 from pydantic import Field, FilePath, RootModel, ValidationError, field_validator, validate_call
+from tomlkit import dumps as toml_dumps
 
 from sereto.enums import Risk
 from sereto.exceptions import SeretoPathError, SeretoValueError
 from sereto.models.base import SeretoBaseModel
-from sereto.models.locator import LocatorModel
-from sereto.types import TypeCategoryName
+from sereto.models.locator import LocatorModel, get_locator_types
+from sereto.sereto_types import TypeCategoryName
 
 
 class VarsMetadataModel(SeretoBaseModel):
@@ -104,15 +104,15 @@ class SubFindingFrontmatterModel(SeretoBaseModel):
             "name": self.name,
             "risk": self.risk.value,
             "category": self.category.lower(),
-            "locators": [locator.model_dump() for locator in self.locators],
+            "locators": [locator.model_dump(exclude_none=True) for locator in self.locators],
         }
         if self.template_path:
             data["template_path"] = self.template_path
-        if self.variables:
+        if len(self.variables) > 0 and any(v is not None for v in self.variables.values()):
             data["variables"] = {k: v for k, v in self.variables.items() if v is not None}
 
         # Dump to TOML string
-        return tomli_w.dumps(data)
+        return toml_dumps(data)
 
     @classmethod
     @validate_call
@@ -132,11 +132,13 @@ class FindingGroupModel(SeretoBaseModel):
         risks: Explicit risks associated with the finding group for specific versions.
         findings: The list of sub-findings in the format of their unique name to include in the report.
         locators: A list of locators used to find the finding group.
+        show_locator_types: A list of locator types to return from the FindingGroup.locators() property.
     """
 
     risk: Risk | None = None
     findings: list[str] = Field(min_length=1)
     locators: list[LocatorModel] = Field(default_factory=list)
+    show_locator_types: list[str] = Field(default_factory=get_locator_types)
 
     @field_validator("risk", mode="before")
     @classmethod
